-- ***************************************************************************************************************************************************
-- * QuestList/Main.lua                                                                                                                              *
-- ***************************************************************************************************************************************************
-- * Gadget file                                                                                                                                     *
-- ***************************************************************************************************************************************************
-- * 0.0.1 / 2012.08.26 / Baanano: First version                                                                                                     *
-- ***************************************************************************************************************************************************

local addonInfo, InternalInterface = ...
local addonID = addonInfo.identifier
_G[addonID] = _G[addonID] or {}
_G[addonID].QuestList = _G[addonID].QuestList or {}
local PublicInterface = _G[addonID].QuestList or {}

local DataGrid = Yague.DataGrid
local Dropdown = Yague.Dropdown
local IUDetail = Inspect.Unit.Detail
local MMax = math.max
local MMin = math.min
local Panel = Yague.Panel
local Popup = Yague.Popup
local ShadowedText = Yague.ShadowedText
local TInsert = table.insert
local UICreateFrame = UI.CreateFrame
local ipairs = ipairs
local pairs = pairs
local setmetatable = setmetatable
local type = type

local questChangeEvent = Utility.Event.Create(addonID, "QuestAvailabilityChange")

local playerFaction = nil
local function PlayerFaction()
	if not playerFaction then
		local detail = IUDetail("player")
		playerFaction = detail and detail.alliance or nil
	end
	return playerFaction
end

local playerLevel = nil
local function PlayerLevel()
	if not playerLevel then
		local detail = IUDetail("player")
		playerLevel = detail and detail.level or nil
	end
	return playerLevel
end
TInsert(Event.Unit.Detail.Level, { function() playerLevel = nil questChangeEvent() end, addonID, addonID .. ".OnUnitLevelChanged" })

local questList = InternalInterface.QuestList.Quests

local inProgressQuests = {}
for questID in pairs(Inspect.Quest.List() or {}) do
	inProgressQuests[questID:sub(1, 9)] = true
end

local completedQuests = {}
for questID in pairs(Inspect.Quest.Complete() or {}) do
	completedQuests[questID:sub(1, 9)] = questList[questID:sub(1, 9)] and questList[questID:sub(1, 9)].repeatable ~= 1 and true or false
end

local function OnQuestAbandon(list)
	for questID in pairs(list) do
		inProgressQuests[questID:sub(1, 9)] = nil
	end
	questChangeEvent()
end
TInsert(Event.Quest.Abandon, { OnQuestAbandon, addonID, addonID .. ".QuestList.OnQuestAbandon" })

local function OnQuestAccept(list)
	for questID in pairs(list) do
		inProgressQuests[questID:sub(1, 9)] = true
	end
	questChangeEvent()
end
TInsert(Event.Quest.Accept, { OnQuestAccept, addonID, addonID .. ".QuestList.OnQuestAccept" })

local function OnQuestComplete(list)
	for questID in pairs(list) do
		questID = questID:sub(1, 9)
		inProgressQuests[questID] = nil
		completedQuests[questID] = questList[questID] and questList[questID].repeatable ~= 1 and true or false
	end
	questChangeEvent()
end
TInsert(Event.Quest.Complete, { OnQuestComplete, addonID, addonID .. ".QuestList.OnQuestComplete" })

local questSource = {}
for id, info in pairs(questList) do
	questSource[id] = { }
	setmetatable(questSource[id], { __index =
		function(t, k)
			if k == "availability" then
				if info.deprecated then return 5 end
				if completedQuests[id] then return 4 end
				if inProgressQuests[id] then return 1 end
				
				for _, questID in ipairs(info.requireNone or {}) do
					if completedQuests[questID] then return 3 end
				end
				
				local any = (not info.requireAny) and true or false
				for _, questID in ipairs(info.requireAny or {}) do
					any = any or completedQuests[questID]
				end
				if not any then return 3 end
				
				for _, questID in pairs(info.requireAll or {}) do
					if not completedQuests[questID] then return 3 end
				end
				
				for _, questID in pairs(info.requireOnNone or {}) do
					if inProgressQuests[questID] then return 3 end
				end
				
				for _, questID in pairs(info.requireOnAll or {}) do
					if not inProgressQuests[questID] then return 3 end
				end
				
				local level = PlayerLevel() or 0
				if level > 0 and info.level and info.level > 0 and info.level - level >= 5 then return 3 end
				
				-- TODO Guild level
				
				local faction = PlayerFaction()
				if faction then
					local oppositeFaction = faction == "defiant" and 3 or (faction == "guardian" and 2 or nil)
					if oppositeFaction and info.factions[oppositeFaction] then return 3 end
				end
				
				return 2
			end
			return info[k]
		end
	})
end

local repeatableLookup = InternalInterface.QuestList.RepeatableLookup
local repeatableSource = { [0] = { displayName = "Any" } }
for id, name in pairs(repeatableLookup) do repeatableSource[id] = { displayName = name } end

local zoneLookup = InternalInterface.QuestList.ZoneLookup
local zoneSource = { [0] = { displayName = "Any" } }
for id, name in pairs(zoneLookup) do zoneSource[id] = { displayName = name } end

local eventLookup = InternalInterface.QuestList.EventLookup
local eventSource = { [0] = { displayName = "Any" } }
for id, name in pairs(eventLookup) do eventSource[id] = { displayName = name } end

local sceneLookup = InternalInterface.QuestList.SceneLookup
local sceneSource = { [0] = { displayName = "Any" } }
for id, name in pairs(sceneLookup) do sceneSource[id] = { displayName = name } end

local scopeLookup = InternalInterface.QuestList.ScopeLookup
local scopeSource = { [0] = { displayName = "Any" } }
for id, name in pairs(scopeLookup) do scopeSource[id] = { displayName = name } end

local typeLookup = InternalInterface.QuestList.TypeLookup
local typeSource = { [0] = { displayName = "Any" } }
for id, name in pairs(typeLookup) do typeSource[id] = { displayName = name } end

local factionLookup = InternalInterface.QuestList.FactionLookup
local factionSource = { [0] = { displayName = "Any" } }
for id, name in pairs(factionLookup) do factionSource[id] = { displayName = name } end

local npcLookup = InternalInterface.QuestList.NPCsLookup

local availabilitySource =
{
	[0] = { displayName = "Any" },
	[1] = { displayName = "In progress" },
	[2] = { displayName = "Available" },
	[3] = { displayName = "Unavailable" },
	[4] = { displayName = "Completed" },
	[5] = { displayName = "Deprecated" },
}

local frameList = {}
function PublicInterface.SearchQuest(frameID, ...)
	if frameList[frameID] then
		frameList[frameID]:SearchQuest(...)
	end
end

local function AvailabilityCellType(name, parent)
	local backgroundCell = UICreateFrame("Frame", name, parent)
	
	function backgroundCell:SetValue(key, value, width, extra)
		self:ClearAll()
		self:SetAllPoints()
		self:SetLayer(self:GetParent():GetLayer() - 1)
		self:SetBackgroundColor(unpack(extra.Color(value)))
	end
	
	return backgroundCell
end

local function CreateGadget(configuration)
	local addonHandle = UICreateFrame("Texture", "Handle", WT.Context)

	local popup = Popup("Popup", WT.Context)
	
	local frame = popup:GetContent()
	
	local frameID = #frameList
	frameList[frameID] = frame
	
	local filterFrame = UICreateFrame("Frame", frame:GetName() .. ".FilterFrame", frame)
	
	local labelQuestName = ShadowedText(filterFrame:GetName() .. ".LabelQuestName", filterFrame)
	local panelQuestName = Panel(filterFrame:GetName() .. ".PanelQuestName", filterFrame)
	local fieldQuestName = UICreateFrame("RiftTextfield", filterFrame:GetName() .. ".FieldQuestName", panelQuestName:GetContent())
	
	local labelRepeatable = ShadowedText(filterFrame:GetName() .. ".LabelRepeatable", filterFrame)
	local dropdownRepeatable = Dropdown(filterFrame:GetName() .. ".DropdownRepeatable", filterFrame)
	
	local labelZone = ShadowedText(filterFrame:GetName() .. ".LabelZone", filterFrame)
	local dropdownZone = Dropdown(filterFrame:GetName() .. ".DropdownZone", filterFrame)
	
	local labelEvent = ShadowedText(filterFrame:GetName() .. ".LabelEvent", filterFrame)
	local dropdownEvent = Dropdown(filterFrame:GetName() .. ".DropdownEvent", filterFrame)
	
	local labelScene = ShadowedText(filterFrame:GetName() .. ".LabelScene", filterFrame)
	local dropdownScene = Dropdown(filterFrame:GetName() .. ".DropdownScene", filterFrame)
	
	local labelType = ShadowedText(filterFrame:GetName() .. ".LabelType", filterFrame)
	local dropdownType = Dropdown(filterFrame:GetName() .. ".DropdownType", filterFrame)
	
	local labelFaction = ShadowedText(filterFrame:GetName() .. ".LabelFaction", filterFrame)
	local dropdownFaction = Dropdown(filterFrame:GetName() .. ".DropdownFaction", filterFrame)
	
	local labelAvailability = ShadowedText(filterFrame:GetName() .. ".LabelAvailability", filterFrame)
	local dropdownAvailability = Dropdown(filterFrame:GetName() .. ".DropdownAvailability", filterFrame)
	
	local labelScope = ShadowedText(filterFrame:GetName() .. ".LabelScope", filterFrame)
	local dropdownScope = Dropdown(filterFrame:GetName() .. ".DropdownScope", filterFrame)
	
	local buttonReset = UICreateFrame("Text", filterFrame:GetName() .. ".ButtonReset", filterFrame)
	
	local gridQuests = DataGrid(frame:GetName() .. ".GridQuests", frame)
	
	local panelInfo = Panel(frame:GetName() .. ".PanelInfo", frame)
	local panelContent = panelInfo:GetContent()

	local scrollbarInfo = UICreateFrame("RiftScrollbar", panelInfo:GetName() .. ".ScrollbarInfo", panelContent)
	local panelInner = Panel(panelInfo:GetName() .. ".PanelInner", panelContent)
	local innerContent = panelInner:GetContent()
	local frameInfo = UICreateFrame("Frame", panelInfo:GetName() .. ".FrameInfo", innerContent)
	
	local labelQuest = ShadowedText(panelInfo:GetName() .. ".LabelQuest", frameInfo)
	local textQuest = UICreateFrame("Text", panelInfo:GetName() .. ".TextQuest", frameInfo)
	
	local labelZonesInfo = ShadowedText(panelInfo:GetName() .. ".LabelZonesInfo", frameInfo)
	local textZonesInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextZonesInfo", frameInfo)
	
	local labelFactionsInfo = ShadowedText(panelInfo:GetName() .. ".LabelFactionsInfo", frameInfo)
	local textFactionsInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextFactionsInfo", frameInfo)
	
	local labelEventsInfo = ShadowedText(panelInfo:GetName() .. ".LabelEventsInfo", frameInfo)
	local textEventsInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextEventsInfo", frameInfo)
	
	local labelScenesInfo = ShadowedText(panelInfo:GetName() .. ".LabelScenesInfo", frameInfo)
	local textScenesInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextScenesInfo", frameInfo)
	
	local labelTypesInfo = ShadowedText(panelInfo:GetName() .. ".LabelTypesInfo", frameInfo)
	local textTypesInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextTypesInfo", frameInfo)
	
	local labelLevel = ShadowedText(panelInfo:GetName() .. ".LabelLevel", frameInfo)
	local textLevel = UICreateFrame("Text", panelInfo:GetName() .. ".TextLevel", frameInfo)
	
	local labelRepeatableInfo = ShadowedText(panelInfo:GetName() .. ".LabelRepeatableInfo", frameInfo)
	local textRepeatableInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextRepeatableInfo", frameInfo)
	
	local labelAvailabilityInfo = ShadowedText(panelInfo:GetName() .. ".LabelAvailabilityInfo", frameInfo)
	local textAvailabilityInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextAvailabilityInfo", frameInfo)
	
	local labelGiversInfo = ShadowedText(panelInfo:GetName() .. ".LabelGiversInfo", frameInfo)
	local textGiversInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextGiversInfo", frameInfo)
	
	local labelCompletersInfo = ShadowedText(panelInfo:GetName() .. ".LabelCompletersInfo", frameInfo)
	local textCompletersInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextCompletersInfo", frameInfo)
	
	local labelScopesInfo = ShadowedText(panelInfo:GetName() .. ".LabelScopesInfo", frameInfo)
	local textScopesInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextScopesInfo", frameInfo)
	
	local labelReqNoneInfo = ShadowedText(panelInfo:GetName() .. ".LabelReqNoneInfo", frameInfo)
	local textReqNoneInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextReqNoneInfo", frameInfo)
	
	local labelReqAnyInfo = ShadowedText(panelInfo:GetName() .. ".LabelReqAnyInfo", frameInfo)
	local textReqAnyInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextReqAnyInfo", frameInfo)
	
	local labelReqAllInfo = ShadowedText(panelInfo:GetName() .. ".LabelReqAllInfo", frameInfo)
	local textReqAllInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextReqAllInfo", frameInfo)
	
	local labelReqOnNoneInfo = ShadowedText(panelInfo:GetName() .. ".LabelReqOnNoneInfo", frameInfo)
	local textReqOnNoneInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextReqOnNoneInfo", frameInfo)
	
	local labelReqOnAllInfo = ShadowedText(panelInfo:GetName() .. ".LabelReqOnAllInfo", frameInfo)
	local textReqOnAllInfo = UICreateFrame("Text", panelInfo:GetName() .. ".TextReqOnAllInfo", frameInfo)
	
	local popupVisible = false
	
	addonHandle:SetTextureAsync("Rift", "indicator_quest_book.png.dds")
	
	popup:SetWidth(820)
	popup:SetHeight(630)
	popup:SetPoint("CENTER", UIParent, "CENTER")
	popup:SetVisible(popupVisible)
	
	filterFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, 10)
	filterFrame:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", -5, 165)
	filterFrame:SetLayer(30)
	
	labelQuestName:SetPoint("CENTERLEFT", filterFrame, 0/9, 1/8, 5, 0)
	labelQuestName:SetFontSize(14)
	labelQuestName:SetFontColor(1, 1, 0.75, 1)
	labelQuestName:SetShadowOffset(2, 2)	
	labelQuestName:SetText("Name:")
	
	panelQuestName:SetPoint("CENTERLEFT", filterFrame, 1/9, 1/8)
	panelQuestName:SetPoint("CENTERRIGHT", filterFrame, 6/9, 1/8)
	panelQuestName:SetHeight(30)
	panelQuestName:SetInvertedBorder(true)
	panelQuestName:GetContent():SetBackgroundColor(0, 0, 0, 0.75)
	
	fieldQuestName:SetPoint("CENTERLEFT", panelQuestName:GetContent(), "CENTERLEFT", 2, 1)
	fieldQuestName:SetPoint("CENTERRIGHT", panelQuestName:GetContent(), "CENTERRIGHT", -2, 1)
	fieldQuestName:SetText("")
	
	labelAvailability:SetPoint("CENTERLEFT", filterFrame, 6/9, 1/8, 5, 0)
	labelAvailability:SetFontSize(14)
	labelAvailability:SetFontColor(1, 1, 0.75, 1)
	labelAvailability:SetShadowOffset(2, 2)	
	labelAvailability:SetText("Availability:")
	
	dropdownAvailability:SetPoint("CENTERLEFT", filterFrame, 7/9, 1/8)
	dropdownAvailability:SetPoint("CENTERRIGHT", filterFrame, 9/9, 1/8)
	dropdownAvailability:SetHeight(30)
	dropdownAvailability:SetTextSelector("displayName")
	dropdownAvailability:SetOrderSelector("")
	dropdownAvailability:SetValues(availabilitySource)
	
	labelZone:SetPoint("CENTERLEFT", filterFrame, 0/9, 3/8, 5, 0)
	labelZone:SetFontSize(14)
	labelZone:SetFontColor(1, 1, 0.75, 1)
	labelZone:SetShadowOffset(2, 2)	
	labelZone:SetText("Zone:")
	
	dropdownZone:SetPoint("CENTERLEFT", filterFrame, 1/9, 3/8)
	dropdownZone:SetPoint("CENTERRIGHT", filterFrame, 6/9, 3/8)
	dropdownZone:SetHeight(30)
	dropdownZone:SetTextSelector("displayName")
	dropdownZone:SetValues(zoneSource)
	
	labelRepeatable:SetPoint("CENTERLEFT", filterFrame, 6/9, 3/8, 5, 0)
	labelRepeatable:SetFontSize(14)
	labelRepeatable:SetFontColor(1, 1, 0.75, 1)
	labelRepeatable:SetShadowOffset(2, 2)	
	labelRepeatable:SetText("Repeatable:")
	
	dropdownRepeatable:SetPoint("CENTERLEFT", filterFrame, 7/9, 3/8)
	dropdownRepeatable:SetPoint("CENTERRIGHT", filterFrame, 9/9, 3/8)
	dropdownRepeatable:SetHeight(30)
	dropdownRepeatable:SetTextSelector("displayName")
	dropdownRepeatable:SetValues(repeatableSource)
	
	labelFaction:SetPoint("CENTERLEFT", filterFrame, 0/9, 5/8, 5, 0)
	labelFaction:SetFontSize(14)
	labelFaction:SetFontColor(1, 1, 0.75, 1)
	labelFaction:SetShadowOffset(2, 2)	
	labelFaction:SetText("Faction:")
	
	dropdownFaction:SetPoint("CENTERLEFT", filterFrame, 1/9, 5/8)
	dropdownFaction:SetPoint("CENTERRIGHT", filterFrame, 3/9, 5/8)
	dropdownFaction:SetHeight(30)
	dropdownFaction:SetTextSelector("displayName")
	dropdownFaction:SetValues(factionSource)

	labelType:SetPoint("CENTERLEFT", filterFrame, 3/9, 5/8, 5, 0)
	labelType:SetFontSize(14)
	labelType:SetFontColor(1, 1, 0.75, 1)
	labelType:SetShadowOffset(2, 2)	
	labelType:SetText("Type:")
	
	dropdownType:SetPoint("CENTERLEFT", filterFrame, 4/9, 5/8)
	dropdownType:SetPoint("CENTERRIGHT", filterFrame, 6/9, 5/8)
	dropdownType:SetHeight(30)
	dropdownType:SetTextSelector("displayName")
	dropdownType:SetValues(typeSource)

	labelScope:SetPoint("CENTERLEFT", filterFrame, 6/9, 5/8, 5, 0)
	labelScope:SetFontSize(14)
	labelScope:SetFontColor(1, 1, 0.75, 1)
	labelScope:SetShadowOffset(2, 2)	
	labelScope:SetText("Scope:")
	
	dropdownScope:SetPoint("CENTERLEFT", filterFrame, 7/9, 5/8)
	dropdownScope:SetPoint("CENTERRIGHT", filterFrame, 9/9, 5/8)
	dropdownScope:SetHeight(30)
	dropdownScope:SetTextSelector("displayName")
	dropdownScope:SetValues(scopeSource)
	
	labelEvent:SetPoint("CENTERLEFT", filterFrame, 0/9, 7/8, 5, 0)
	labelEvent:SetFontSize(14)
	labelEvent:SetFontColor(1, 1, 0.75, 1)
	labelEvent:SetShadowOffset(2, 2)	
	labelEvent:SetText("Event:")
	
	dropdownEvent:SetPoint("CENTERLEFT", filterFrame, 1/9, 7/8)
	dropdownEvent:SetPoint("CENTERRIGHT", filterFrame, 4/9, 7/8)
	dropdownEvent:SetHeight(30)
	dropdownEvent:SetTextSelector("displayName")
	dropdownEvent:SetValues(eventSource)
	
	labelScene:SetPoint("CENTERLEFT", filterFrame, 4/9, 7/8, 5, 0)
	labelScene:SetFontSize(14)
	labelScene:SetFontColor(1, 1, 0.75, 1)
	labelScene:SetShadowOffset(2, 2)	
	labelScene:SetText("Scene:")
	
	dropdownScene:SetPoint("CENTERLEFT", filterFrame, 5/9, 7/8)
	dropdownScene:SetPoint("CENTERRIGHT", filterFrame, 8/9, 7/8)
	dropdownScene:SetHeight(30)
	dropdownScene:SetTextSelector("displayName")
	dropdownScene:SetValues(sceneSource)
	
	buttonReset:SetPoint("CENTER", filterFrame, 17/18, 7/8, 0, 0)
	buttonReset:SetText("<u>Reset</u>", true)
	buttonReset:SetFontColor(1, 0.5, 0)
	
	local function AvailabilityColor(value)
		local r, g, b = 0, 0, 1 -- Deprecated
		if value.availability == 1 then -- In progress
			r, g, b = 1, 0, 1
		elseif value.availability == 2 then -- Available
			local levelDiff = (value.level or 0) - (PlayerLevel() or 0)
			if levelDiff > 4 then
				r, g, b = 1, 0, 0 -- Red
			elseif levelDiff > 2 then
				r, g, b = 1, 0.5, 0 -- Orange
			elseif levelDiff > -3 then
				r, g, b = 1, 1, 0 -- Yellow
			elseif levelDiff > -5 then
				r, g, b = 0, 1, 0 -- Green
			else
				r, g, b = 0, 0, 0 -- Grey
			end
		elseif value.availability == 3 then -- Unavailable
			r, g, b = 1, 0, 0
		elseif value.availability == 4 then -- Completed
			r, g, b = 0, 1, 1
		end
		
		return { r, g, b, 0.1 }
	end
	
	gridQuests:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, 170)
	gridQuests:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -5, -210)
	gridQuests:SetHeadersVisible(false)
	gridQuests:SetRowHeight(20)
	gridQuests:SetRowMargin(0)
	gridQuests:SetUnselectedRowBackgroundColor({0.2, 0.2, 0.2, 0.25})
	gridQuests:SetSelectedRowBackgroundColor({0.6, 0.6, 0.6, 0.25})
	gridQuests:AddColumn("name", "Quest Name", "Text", 350, 1, "name", true, { Alignment = "left", Formatter = "none" })
	gridQuests:AddColumn("background", nil, AvailabilityCellType, 0, 0, nil, false, { Color = AvailabilityColor })
	gridQuests:SetFilter(
		function(questID, questInfo)
			local zone = dropdownZone:GetSelectedValue() or 0
			if zone > 0 and not questInfo.zones[zone] then return false end

			local repeatable = dropdownRepeatable:GetSelectedValue() or 0
			if repeatable > 0 and questInfo.repeatable ~= repeatable then return false end

			local faction = dropdownFaction:GetSelectedValue() or 0
			if faction > 0 and not questInfo.factions[faction] then return false end

			local qType = dropdownType:GetSelectedValue() or 0
			if qType > 0 and not questInfo.types[qType] then return false end

			local scope = dropdownScope:GetSelectedValue() or 0
			if scope > 0 and not questInfo.scopes[scope] then return false end

			local event = dropdownEvent:GetSelectedValue() or 0
			if event > 0 and not questInfo.events[event] then return false end

			local scene = dropdownScene:GetSelectedValue() or 0
			if scene > 0 and not questInfo.scenes[scene] then return false end
			
			if not questInfo.name:upper():find(fieldQuestName:GetText():upper(), nil, true) then return false end
			
			local availability = dropdownAvailability:GetSelectedValue() or 0
			if availability > 0 and questInfo.availability ~= availability then return false end

			return true
		end)
	gridQuests:SetData(questSource)
	gridQuests:SetLayer(10)
	
	panelInfo:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 5, -205)
	panelInfo:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -5, -10)
	panelInfo:SetLayer(20)
	
	scrollbarInfo:SetPoint("TOPLEFT", panelContent, "TOPRIGHT", -18, 1)
	scrollbarInfo:SetPoint("BOTTOMRIGHT", panelContent, "BOTTOMRIGHT", -2, -1)
	
	panelInner:SetPoint("TOPLEFT", panelContent, "TOPLEFT", 1, 1)
	panelInner:SetPoint("BOTTOMRIGHT", panelContent, "BOTTOMRIGHT", -21, -1)
	panelInner:SetInvertedBorder(true)
	
	innerContent:SetBackgroundColor(0, 0, 0.05, 0.75)
	
	frameInfo:SetPoint("TOPCENTER", innerContent, "TOPCENTER", 0, 0)
	frameInfo:SetWidth(innerContent:GetWidth())
	
	labelQuest:SetPoint("CENTERLEFT", frameInfo, "TOPLEFT", 5, 15)
	labelQuest:SetFontSize(14)
	labelQuest:SetFontColor(1, 1, 0.75, 1)
	labelQuest:SetShadowOffset(2, 2)	
	labelQuest:SetText("Quest:")

	textQuest:SetPoint("CENTERLEFT", frameInfo, "TOPLEFT", 90, 15)

	textZonesInfo:SetPoint("TOPLEFT", textQuest, "BOTTOMLEFT", 0, 10)

	labelZonesInfo:SetPoint("LEFT", labelQuest, "LEFT")
	labelZonesInfo:SetPoint("CENTERRIGHT", textZonesInfo, "CENTERLEFT")
	labelZonesInfo:SetFontSize(14)
	labelZonesInfo:SetFontColor(1, 1, 0.75, 1)
	labelZonesInfo:SetShadowOffset(2, 2)	
	labelZonesInfo:SetText("Zones:")

	textFactionsInfo:SetPoint("TOPLEFT", textZonesInfo, "BOTTOMLEFT", 0, 10)

	labelFactionsInfo:SetPoint("LEFT", labelQuest, "LEFT")
	labelFactionsInfo:SetPoint("CENTERRIGHT", textFactionsInfo, "CENTERLEFT")
	labelFactionsInfo:SetFontSize(14)
	labelFactionsInfo:SetFontColor(1, 1, 0.75, 1)
	labelFactionsInfo:SetShadowOffset(2, 2)	
	labelFactionsInfo:SetText("Factions:")

	textEventsInfo:SetPoint("TOPLEFT", textFactionsInfo, "BOTTOMLEFT", 0, 10)

	labelEventsInfo:SetPoint("LEFT", labelQuest, "LEFT")
	labelEventsInfo:SetPoint("CENTERRIGHT", textEventsInfo, "CENTERLEFT")
	labelEventsInfo:SetFontSize(14)
	labelEventsInfo:SetFontColor(1, 1, 0.75, 1)
	labelEventsInfo:SetShadowOffset(2, 2)	
	labelEventsInfo:SetText("Events:")

	textScenesInfo:SetPoint("TOPLEFT", textEventsInfo, "BOTTOMLEFT", 0, 10)

	labelScenesInfo:SetPoint("LEFT", labelQuest, "LEFT")
	labelScenesInfo:SetPoint("CENTERRIGHT", textScenesInfo, "CENTERLEFT")
	labelScenesInfo:SetFontSize(14)
	labelScenesInfo:SetFontColor(1, 1, 0.75, 1)
	labelScenesInfo:SetShadowOffset(2, 2)	
	labelScenesInfo:SetText("Scenes:")

	textTypesInfo:SetPoint("TOPLEFT", textScenesInfo, "BOTTOMLEFT", 0, 10)

	labelTypesInfo:SetPoint("LEFT", labelQuest, "LEFT")
	labelTypesInfo:SetPoint("CENTERRIGHT", textTypesInfo, "CENTERLEFT")
	labelTypesInfo:SetFontSize(14)
	labelTypesInfo:SetFontColor(1, 1, 0.75, 1)
	labelTypesInfo:SetShadowOffset(2, 2)	
	labelTypesInfo:SetText("Types:")
	
	labelLevel:SetPoint("CENTERLEFT", frameInfo, "TOPRIGHT", -300, 15)
	labelLevel:SetFontSize(14)
	labelLevel:SetFontColor(1, 1, 0.75, 1)
	labelLevel:SetShadowOffset(2, 2)	
	labelLevel:SetText("Level:")

	textLevel:SetPoint("CENTERRIGHT", frameInfo, "TOPRIGHT", -5, 15)

	textRepeatableInfo:SetPoint("TOPRIGHT", textLevel, "BOTTOMRIGHT", 0, 10)

	labelRepeatableInfo:SetPoint("LEFT", labelLevel, "LEFT")
	labelRepeatableInfo:SetPoint("CENTERRIGHT", textRepeatableInfo, "CENTERLEFT")
	labelRepeatableInfo:SetFontSize(14)
	labelRepeatableInfo:SetFontColor(1, 1, 0.75, 1)
	labelRepeatableInfo:SetShadowOffset(2, 2)	
	labelRepeatableInfo:SetText("Repeatable:")

	textAvailabilityInfo:SetPoint("TOPRIGHT", textRepeatableInfo, "BOTTOMRIGHT", 0, 10)
	
	labelAvailabilityInfo:SetPoint("LEFT", labelLevel, "LEFT")
	labelAvailabilityInfo:SetPoint("CENTERRIGHT", textAvailabilityInfo, "CENTERLEFT")
	labelAvailabilityInfo:SetFontSize(14)
	labelAvailabilityInfo:SetFontColor(1, 1, 0.75, 1)
	labelAvailabilityInfo:SetShadowOffset(2, 2)	
	labelAvailabilityInfo:SetText("Availability:")

	textGiversInfo:SetPoint("TOPRIGHT", textAvailabilityInfo, "BOTTOMRIGHT", 0, 10)
	
	labelGiversInfo:SetPoint("LEFT", labelLevel, "LEFT")
	labelGiversInfo:SetPoint("CENTERRIGHT", textGiversInfo, "CENTERLEFT")
	labelGiversInfo:SetFontSize(14)
	labelGiversInfo:SetFontColor(1, 1, 0.75, 1)
	labelGiversInfo:SetShadowOffset(2, 2)	
	labelGiversInfo:SetText("Givers:")

	textCompletersInfo:SetPoint("TOPRIGHT", textGiversInfo, "BOTTOMRIGHT", 0, 10)
	
	labelCompletersInfo:SetPoint("LEFT", labelLevel, "LEFT")
	labelCompletersInfo:SetPoint("CENTERRIGHT", textCompletersInfo, "CENTERLEFT")
	labelCompletersInfo:SetFontSize(14)
	labelCompletersInfo:SetFontColor(1, 1, 0.75, 1)
	labelCompletersInfo:SetShadowOffset(2, 2)	
	labelCompletersInfo:SetText("Completers:")

	textScopesInfo:SetPoint("TOPRIGHT", textCompletersInfo, "BOTTOMRIGHT", 0, 10)
	
	labelScopesInfo:SetPoint("LEFT", labelLevel, "LEFT")
	labelScopesInfo:SetPoint("CENTERRIGHT", textScopesInfo, "CENTERLEFT")
	labelScopesInfo:SetFontSize(14)
	labelScopesInfo:SetFontColor(1, 1, 0.75, 1)
	labelScopesInfo:SetShadowOffset(2, 2)	
	labelScopesInfo:SetText("Scopes:")
	
	textReqNoneInfo:SetPoint("TOPRIGHT", frameInfo, "TOPRIGHT", -5, 200)

	labelReqNoneInfo:SetPoint("LEFT", labelQuest, "LEFT")
	labelReqNoneInfo:SetPoint("CENTERRIGHT", textReqNoneInfo, "CENTERLEFT")
	labelReqNoneInfo:SetFontSize(14)
	labelReqNoneInfo:SetFontColor(1, 1, 0.75, 1)
	labelReqNoneInfo:SetShadowOffset(2, 2)	
	labelReqNoneInfo:SetText("Requires (none completed):")
	
	textReqAnyInfo:SetPoint("TOPRIGHT", textReqNoneInfo, "BOTTOMRIGHT", 0, 10)

	labelReqAnyInfo:SetPoint("LEFT", labelQuest, "LEFT")
	labelReqAnyInfo:SetPoint("CENTERRIGHT", textReqAnyInfo, "CENTERLEFT")
	labelReqAnyInfo:SetFontSize(14)
	labelReqAnyInfo:SetFontColor(1, 1, 0.75, 1)
	labelReqAnyInfo:SetShadowOffset(2, 2)	
	labelReqAnyInfo:SetText("Requires (any completed):")
	
	textReqAllInfo:SetPoint("TOPRIGHT", textReqAnyInfo, "BOTTOMRIGHT", 0, 10)

	labelReqAllInfo:SetPoint("LEFT", labelQuest, "LEFT")
	labelReqAllInfo:SetPoint("CENTERRIGHT", textReqAllInfo, "CENTERLEFT")
	labelReqAllInfo:SetFontSize(14)
	labelReqAllInfo:SetFontColor(1, 1, 0.75, 1)
	labelReqAllInfo:SetShadowOffset(2, 2)	
	labelReqAllInfo:SetText("Requires (all completed):")
	
	textReqOnNoneInfo:SetPoint("TOPRIGHT", textReqAllInfo, "BOTTOMRIGHT", 0, 10)

	labelReqOnNoneInfo:SetPoint("LEFT", labelQuest, "LEFT")
	labelReqOnNoneInfo:SetPoint("CENTERRIGHT", textReqOnNoneInfo, "CENTERLEFT")
	labelReqOnNoneInfo:SetFontSize(14)
	labelReqOnNoneInfo:SetFontColor(1, 1, 0.75, 1)
	labelReqOnNoneInfo:SetShadowOffset(2, 2)	
	labelReqOnNoneInfo:SetText("Requires (none in progress):")
	
	textReqOnAllInfo:SetPoint("TOPRIGHT", textReqOnNoneInfo, "BOTTOMRIGHT", 0, 10)

	labelReqOnAllInfo:SetPoint("LEFT", labelQuest, "LEFT")
	labelReqOnAllInfo:SetPoint("CENTERRIGHT", textReqOnAllInfo, "CENTERLEFT")
	labelReqOnAllInfo:SetFontSize(14)
	labelReqOnAllInfo:SetFontColor(1, 1, 0.75, 1)
	labelReqOnAllInfo:SetShadowOffset(2, 2)	
	labelReqOnAllInfo:SetText("Requires (all in progress):")

	function addonHandle.Event:LeftClick()
		popupVisible = not popupVisible
		if popupVisible then
			WT.FadeIn(popup, 0.5)
		else
			WT.FadeOut(popup, 0.5)
		end
	end
	
	function panelQuestName.Event:LeftClick()
		fieldQuestName:SetKeyFocus(true)
	end

	function fieldQuestName.Event:KeyFocusGain()
		local length = self:GetText():len()
		if length > 0 then
			self:SetSelection(0, length)
		end
	end
	
	local dontRefresh = nil
	local function RefreshFilter() if not dontRefresh then gridQuests:RefreshFilter() end end
	fieldQuestName.Event.TextfieldChange = RefreshFilter
	dropdownAvailability.Event.SelectionChanged = RefreshFilter
	dropdownZone.Event.SelectionChanged = RefreshFilter
	dropdownRepeatable.Event.SelectionChanged = RefreshFilter
	dropdownFaction.Event.SelectionChanged = RefreshFilter
	dropdownType.Event.SelectionChanged = RefreshFilter
	dropdownScope.Event.SelectionChanged = RefreshFilter
	dropdownEvent.Event.SelectionChanged = RefreshFilter
	dropdownScene.Event.SelectionChanged = RefreshFilter
	
	function panelInfo.Event:WheelForward()
		local position = scrollbarInfo:GetPosition()
		local minRange, maxRange = scrollbarInfo:GetRange()
		position = MMax(position - 20, minRange)
		scrollbarInfo:SetPosition(position)
	end
	
	function panelInfo.Event:WheelBack()
		local position = scrollbarInfo:GetPosition() + 20
		local minRange, maxRange = scrollbarInfo:GetRange()
		position = MMin(position + 20, maxRange)
		scrollbarInfo:SetPosition(position)
	end
	
	function scrollbarInfo.Event:ScrollbarChange()
		frameInfo:SetPoint("TOPCENTER", innerContent, "TOPCENTER", 0, -self:GetPosition())
	end
	
	function gridQuests.Event:SelectionChanged(questID, questInfo)
		textQuest:SetText("", false)
		textZonesInfo:SetText("", false)
		textFactionsInfo:SetText("", false)
		textEventsInfo:SetText("", false)
		textScenesInfo:SetText("", false)
		textTypesInfo:SetText("", false)
		textLevel:SetText("", false)
		textRepeatableInfo:SetText("", false)
		textAvailabilityInfo:SetText("", false)
		textGiversInfo:SetText("", false)
		textCompletersInfo:SetText("", false)
		textScopesInfo:SetText("", false)
		textReqNoneInfo:SetText("", false)
		textReqAnyInfo:SetText("", false)
		textReqAllInfo:SetText("", false)
		textReqOnNoneInfo:SetText("", false)
		textReqOnAllInfo:SetText("", false)

		if questInfo then
			textQuest:SetText(questInfo.name or "", false)

			local zones = ""
			for id in pairs(questInfo.zones) do zones = zones .. "<u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { zoneID = " .. id .. " })'>" .. zoneLookup[id] .. "</a></u>\n" end
			textZonesInfo:SetText(zones, true)
			
			local factions = ""
			for id in pairs(questInfo.factions) do factions = factions .. "<u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { factionID = " .. id .. " })'>" .. factionLookup[id] .. "</a></u>\n" end
			textFactionsInfo:SetText(factions, true)

			local events = ""
			for id in pairs(questInfo.events) do events = events .. "<u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { eventID = " .. id .. " })'>" .. eventLookup[id] .. "</a></u>\n" end
			textEventsInfo:SetText(events, true)

			local scenes = ""
			for id in pairs(questInfo.scenes) do scenes = scenes .. "<u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { sceneID = " .. id .. " })'>" .. sceneLookup[id] .. "</a></u>\n" end
			textScenesInfo:SetText(scenes, true)
			
			local types = ""
			for id in pairs(questInfo.types) do types = types .. "<u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { typeID = " .. id .. " })'>" .. typeLookup[id] .. "</a></u>\n" end
			textTypesInfo:SetText(types, true)
			
			local level = questInfo.level and tostring(questInfo.level) or ""
			if questInfo.guildLevel then
				level = (level == "" and "" or (level .. " / ")) .. questInfo.guildLevel .. " (guild)"
			end
			textLevel:SetText(level == "" and "Any" or level, false)
			
			textRepeatableInfo:SetText("<u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { repeatableID = " .. questInfo.repeatable .. " })'>" .. repeatableLookup[questInfo.repeatable] .. "</a></u>", true)
			
			textAvailabilityInfo:SetText("<u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { availabilityID = " .. questInfo.availability .. " })'>" .. availabilitySource[questInfo.availability].displayName .. "</a></u>", true)
			
			local givers = ""
			if type(questInfo.npcGivers) == "table" then for _, npcID in ipairs(questInfo.npcGivers) do givers = givers .. npcLookup[npcID] .. "\n" end end
			givers = givers == "" and "Unknown" or givers
			textGiversInfo:SetText(givers, false)

			local completers = ""
			if type(questInfo.npcCompleters) == "table" then for _, npcID in ipairs(questInfo.npcCompleters) do completers = completers .. npcLookup[npcID] .. "\n" end end
			completers = completers == "" and "Unknown" or completers
			textCompletersInfo:SetText(completers, false)
			
			local scopes = ""
			for id in pairs(questInfo.scopes) do scopes = scopes .. "<u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { scopeID = " .. id .. " })'>" .. scopeLookup[id] .. "</a></u>\n" end
			textScopesInfo:SetText(scopes, true)

			local reqNone = ""
			if type(questInfo.requireNone) == "table" then
				for _, id in ipairs(questInfo.requireNone) do
					if questList[id] then
						reqNone = reqNone .. "<font color=\"#" .. (completedQuests[id] and "FF0000" or "00FF00")  .. "\"><u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { name = \"" .. questList[id].name:gsub("'", "-DASH") .. "\" })'>" .. questList[id].name .. "</a></u></font>\n"
					end
				end
			end
			labelReqNoneInfo:SetVisible(reqNone ~= "")
			textReqNoneInfo:SetText(reqNone, true)
			
			local reqAny = ""
			if type(questInfo.requireAny) == "table" then
				for _, id in ipairs(questInfo.requireAny) do
					if questList[id] then
						reqAny = reqAny .. "<font color=\"#" .. (completedQuests[id] and "00FF00" or "FF0000")  .. "\"><u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { name = \"" .. questList[id].name:gsub("'", "-DASH") .. "\" })'>" .. questList[id].name .. "</a></u></font>\n"
					end
				end
			end
			labelReqAnyInfo:SetVisible(reqAny ~= "")
			textReqAnyInfo:SetText(reqAny, true)

			local reqAll = ""
			if type(questInfo.requireAll) == "table" then
				for _, id in ipairs(questInfo.requireAll) do
					if questList[id] then
						reqAll = reqAll .. "<font color=\"#" .. (completedQuests[id] and "00FF00" or "FF0000")  .. "\"><u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { name = \"" .. questList[id].name:gsub("'", "-DASH") .. "\" })'>" .. questList[id].name .. "</a></u></font>\n"
					end
				end
			end
			labelReqAllInfo:SetVisible(reqAll ~= "")
			textReqAllInfo:SetText(reqAll, true)

			local reqOnNone = ""
			if type(questInfo.requireOnNone) == "table" then
				for _, id in ipairs(questInfo.requireOnNone) do
					if questList[id] then
						reqOnNone = reqOnNone .. "<font color=\"#" .. (inProgressQuests[id] and "FF0000" or "00FF00")  .. "\"><u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { name = \"" .. questList[id].name:gsub("'", "-DASH") .. "\" })'>" .. questList[id].name .. "</a></u></font>\n"
					end
				end
			end
			labelReqOnNoneInfo:SetVisible(reqOnNone ~= "")
			textReqOnNoneInfo:SetText(reqOnNone, true)

			local reqOnAll = ""
			if type(questInfo.requireOnAll) == "table" then
				for _, id in ipairs(questInfo.requireOnAll) do
					if questList[id] then
						reqOnAll = reqOnAll .. "<font color=\"#" .. (inProgressQuests[id] and "00FF00" or "FF0000")  .. "\"><u><a lua='" .. addonID .. ".QuestList.SearchQuest(" .. frameID .. ", { name = \"" .. questList[id].name:gsub("'", "-DASH") .. "\" })'>" .. questList[id].name .. "</a></u></font>\n"
					end
				end
			end
			labelReqOnAllInfo:SetVisible(reqOnAll ~= "")
			textReqOnAllInfo:SetText(reqOnAll, true)
		end

		local maxHeight = MMax(textTypesInfo:GetTop() + textTypesInfo:GetHeight(), textScopesInfo:GetTop() + textScopesInfo:GetHeight()) - frameInfo:GetTop() + 10
		textReqNoneInfo:SetPoint("TOPRIGHT", frameInfo, "TOPRIGHT", -5, maxHeight)
		
		maxHeight = MMax(MMax(labelReqOnAllInfo:GetTop() + labelReqOnAllInfo:GetHeight(), textReqOnAllInfo:GetTop() + textReqOnAllInfo:GetHeight()) - frameInfo:GetTop(), innerContent:GetHeight())
		frameInfo:SetHeight(maxHeight)
		scrollbarInfo:SetRange(0, maxHeight - innerContent:GetHeight())
		scrollbarInfo:SetPosition(0)
	end
	
	function buttonReset.Event:LeftClick()
		frame:SearchQuest()
	end
	
	function frame:SearchQuest(params)
		params = params or {}
		
		dontRefresh = true
		dropdownAvailability:SetSelectedKey(params.availabilityID or 0)
		dropdownZone:SetSelectedKey(params.zoneID or 0)
		dropdownRepeatable:SetSelectedKey(params.repeatableID or 0)
		dropdownFaction:SetSelectedKey(params.factionID or 0)
		dropdownType:SetSelectedKey(params.typeID or 0)
		dropdownScope:SetSelectedKey(params.scopeID or 0)
		dropdownEvent:SetSelectedKey(params.eventID or 0)
		dropdownScene:SetSelectedKey(params.sceneID or 0)
		fieldQuestName:SetText(((params.name or ""):gsub("-DASH", "'")))
		dontRefresh = false
		
		RefreshFilter()
	end
	
	TInsert(Event[addonID].QuestAvailabilityChange, { RefreshFilter, addonID, frame:GetName() .. ".OnQuestAvailabilityChange" })
	
	return addonHandle
end

WT.Gadget.RegisterFactory(addonID .. ".QuestList",
{
    name = "Quest List",
    description = "List of known quests",
    author = "Baanano",
    version ="0.0.1",
	iconTexAddon = "Rift",
	iconTexFile = "indicator_quest_book.png.dds",
    ["Create"] = CreateGadget,
})